package com.bwie.auth.service.impl;

import cn.hutool.crypto.digest.MD5;
import com.alibaba.fastjson.JSON;
import com.bwie.auth.feign.UserFeign;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.response.JwtResponse;
import com.bwie.common.exception.AbnormalException;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Service
public class AuthServiceImpl implements AuthService {

    @Autowired
    private UserFeign userFeign;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Override
    public JwtResponse login(User user) {
        Result<User> userResult = userFeign.selectByTel(user.getUserTel());
        if (200!=userResult.getCode()){
            throw new AbnormalException("网络异常");
        }
        if (userResult.getData()==null){
            throw new AbnormalException("手机号不存在");
        }
        MD5 md5 = new MD5();
        String userPwd = md5.digestHex(user.getUserPwd());
        if (!userResult.getData().getUserPwd().equals(userPwd)){
            throw new AbnormalException("密码错误");
        }
        String userKey = UUID.randomUUID().toString();
        HashMap<String, Object> map = new HashMap<>();
        map.put(JwtConstants.USER_KEY,userKey);
        map.put(JwtConstants.DETAILS_USER_ID,userResult.getData().getUserId());
        String token = JwtUtils.createToken(map);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+userKey,
                JSON.toJSONString(userResult.getData()),60, TimeUnit.MINUTES);
        JwtResponse jwtResponse = new JwtResponse();
        jwtResponse.setToken(token);
        jwtResponse.setExistTime("60MIN");
        return jwtResponse;
    }

    /**
     * 获取用户信息
     */
    @Override
    public User info(HttpServletRequest request) {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String userJson = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(userJson, User.class);
        return user;
    }
}
