package com.bwie.consumer;

import com.alibaba.fastjson.JSON;
import com.bwie.common.domain.Record;
import com.bwie.common.domain.User;
import com.bwie.common.result.Result;
import com.bwie.common.utils.SendMessageUtil;
import com.bwie.feign.UserFeign;
import com.rabbitmq.client.Channel;
import lombok.extern.java.Log;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.io.IOException;

@Component
@Log4j2
public class SMSConsumer {

    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private UserFeign userFeign;
    @RabbitListener(queuesToDeclare = @Queue("sendSMS"))
    public void sendSMS(String jsonRecord, Message message, Channel channel){
        String messageId = message.getMessageProperties().getMessageId();
        Long num = redisTemplate.opsForSet().add("sendSMS", messageId);
        if (num>0){
            Record record = JSON.parseObject(jsonRecord, Record.class);
            Result<User> userResult = userFeign.findById(record.getUserId());
            SendMessageUtil sendMessageUtil = new SendMessageUtil();
            String code = "自行车信息"+record.getRecordsId()+",时长"+record.getDuration()+"，费用信息"+record.getExpense();
            log.info("信息{}",code);
            sendMessageUtil.sendSms(userResult.getData().getUserTel(),code);
            log.info("信息发送成功");
            try {
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
            } catch (IOException e) {
                try {
                    redisTemplate.opsForSet().remove("sendSMS", messageId);
                    channel.basicReject(message.getMessageProperties().getDeliveryTag(),true);
                } catch (IOException ex) {
                    throw new RuntimeException(ex);
                }
            }
        }
    }



}
