package com.bwie.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Record;
import com.bwie.common.domain.TypeEnum;
import com.bwie.common.domain.response.RecordResp;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.feign.UserFeign;
import com.bwie.mapper.RecordMapper;
import com.bwie.service.BicycleService;
import com.bwie.service.RecordService;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.core.Local;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.LocalTime;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@Service
public class RecordServiceImpl implements RecordService {
    @Autowired
    private RecordMapper recordMapper;

    @Autowired
    private BicycleService bicycleService;

    @Autowired
    private HttpServletRequest request;

    @Autowired
    private UserFeign userFeign;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    /**
     * 添加记录表
     * @param bicycleId
     * @return
     */
    @Transactional
    @Override
    public int addRecord(Integer bicycleId) {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userId = JwtUtils.getUserId(token);
        Record record = new Record();
        record.setBicycleId(bicycleId);
        record.setUserId(Integer.valueOf(userId));
        record.setStartTime(new Date());
        bicycleService.updState(bicycleId,2);
        return recordMapper.insertRecord(record);
    }
    /**
     * 查询记录表
     * @return
     */
    @Override
    public List<RecordResp> findList() {
        return recordMapper.findList();
    }

    /**
     * 结束行程
     * @param record
     * @return
     */
    @Transactional
    @Override
    public int finish(Record record) {
        Date startTime = record.getStartTime();
        long now = new Date().getTime();
        long start = startTime.getTime();
        //时长
        int time = (int) ((now-start)/1000/60);
        record.setDuration(time);
        //公里数
        double distance = time * 0.8;
        record.setDistance(distance);
        //通过枚举获取每种类型的单价
        BigDecimal price = TypeEnum.price(record.getTypeId());
        //车型公里单价）×公里数+0.1×用车时间（单位分钟）
        BigDecimal multiply = price.multiply(BigDecimal.valueOf(distance));
        double useTime = distance * 0.1;
        BigDecimal expense = multiply.add(BigDecimal.valueOf(useTime));
        record.setExpense(expense);
        Result result = userFeign.updUserMoney(expense, record.getUserId());
        if (200!=result.getCode()){
            throw new ArithmeticException("失败");
        }
        bicycleService.updState(record.getBicycleId(),1);
        int i = recordMapper.updRecord(record);
        if (i>0){
            rabbitTemplate.convertAndSend("sendSMS", JSON.toJSONString(record),message -> {
                String string = UUID.randomUUID().toString();
                message.getMessageProperties().setMessageId(string);
                return message;
            });
        }
        return i;
    }
}
